import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Boilerplate } from 'src/app/models/boilerplate.model';
import { Sections } from 'src/app/models/boilerplate.model';
import { BoilerplateService } from 'src/app/services/boilerplate.service';
import { MainService } from 'src/app/services/main.service';
import { MainDetailsComponent } from 'src/app/components/main-details/main-details.component'
import { PrefixDetailsComponent } from 'src/app/components/prefix-details/prefix-details.component'
import { SuffixDetailsComponent } from 'src/app/components/suffix-details/suffix-details.component'
import { NotifierService } from "angular-notifier";
import { MatDialog, MatDialogConfig } from '@angular/material/dialog';
import { CourseDialogComponentComponent, DialogData } from 'src/app/components/course-dialog-component/course-dialog-component.component';

@Component({
  selector: 'app-boilerplate-details',
  templateUrl: './boilerplate-details.component.html',
  styleUrls: ['./boilerplate-details.component.scss']

})
export class BoilerplateDetailsComponent implements OnInit {
  panelOpenState = false;

  subjects: string[] = [];

  systems?: Sections[];

  currentboilerplate: Boilerplate = {
    group_of_boilerplate: this.route.snapshot.params.gb,
    title: "",
    has_prefix: false,
    has_main: false,
    has_suffix: false
  };


  message = '';
  searchText = '';
  searchTextinstances = '';
  addinstance = '';

  private readonly notifier: NotifierService;



  constructor(
    private mainService: MainService,
    private boilerplateService: BoilerplateService,
    private route: ActivatedRoute,
    notifierService: NotifierService,
    private router: Router,
    public dialog: MatDialog) { this.notifier = notifierService; }

  @ViewChild(MainDetailsComponent) mainchild!: MainDetailsComponent;
  @ViewChild(PrefixDetailsComponent) prefchild!: PrefixDetailsComponent;
  @ViewChild(SuffixDetailsComponent) sufchild!: SuffixDetailsComponent;

  ngOnInit(): void {
    this.message = '';
    this.getchoices(this.route.snapshot.params.gb);
    this.getBoilerplate(this.route.snapshot.params.gb, this.route.snapshot.params.id);
  }


  getBoilerplate(gb: any, id: number): void {
    this.boilerplateService.get(gb, id)
      .subscribe(
        data => {
          this.currentboilerplate = data;
          //console.log(data);
        },
        error => {
          console.log(error);
        });
  }

  updateBoilerplate(): any {
    if (this.currentboilerplate.title != "") {
      this.currentboilerplate.title = this.currentboilerplate.title?.replace(/[^\w\s]/gi, ' ')
      this.currentboilerplate.title = this.currentboilerplate.title?.replace(/ /g, "_")
    }

    this.boilerplateService.update(this.route.snapshot.params.gb, this.currentboilerplate.id, this.currentboilerplate)
      .subscribe(
        response => {
          //this.notifier.notify("success",(JSON.stringify("Requirement " + this.currentboilerplate.title + " updated")));
          //this.router.navigate(['/groupboilerplates/'+this.route.snapshot.params.gb+'/boilerplates']);
          if (this.currentboilerplate.has_prefix) { this.prefchild.getAllPrefix(this.currentboilerplate.id) };
          if (this.currentboilerplate.has_suffix) { this.sufchild.getAllSuffix(this.currentboilerplate.id); };
        },
        error => {
          this.notifier.notify("warning", (JSON.stringify(error)));
        });
  }

  updateDetailsOfBoilerplate(): any {
    if (this.currentboilerplate.has_prefix) { this.prefchild.updatePrefix() };
    if (this.currentboilerplate.has_main) { this.mainchild.updateMain(); };
    if (this.currentboilerplate.has_suffix) { this.sufchild.updateSuffix(); };
    this.notifier.notify("success", (JSON.stringify("Requirement " + this.currentboilerplate.title + " updated")));
  }

  deleteBoilerplate(): void {
    if (confirm("Αre you sure you want to delete this Boilerplate?")) {
      this.boilerplateService.delete(this.route.snapshot.params.gb, this.currentboilerplate.id)
        .subscribe(
          response => {
            //console.log(response);
            this.router.navigate(['/groupboilerplates/' + this.route.snapshot.params.gb + '/boilerplates']);
          },
          error => {
            console.log(error);
          });
    }
  }

  getchoices(gb: any): void {
    //System
    this.mainService.getdictionary(gb)
      .subscribe(
        data => {
          // console.log(data[0].Dictionary_Data);
          this.systems = data[0].Dictionary_Data

        },
        error => {
          console.log(error);
        });
  }

  checklen(i: any): any {
    if (i) {
      if (i.length >= 1) { return true }
    }
    return false
  }

  currentDialog: DialogData = {
    instance: "",
    comment: "",
  }

  openDialog(section: any, classes: any, subclass: any) {
    const dialogRef = this.dialog.open(CourseDialogComponentComponent, {
      width: '500px',
      data: { name: section + (classes ? ' : ' + classes : '') + (subclass ? ' : ' + subclass : ''), instance: this.currentDialog.instance, comment: this.currentDialog.comment },
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result.instance != "") {
        result.instance = result.instance?.replace(/[^\w\s]/gi, ' ')
        result.instance = result.instance?.replace(/ /g, "_")
      }
      if (result.instance) {
        this.addinstance = section + (classes ? ' : ' + classes : '') + (subclass ? ' : ' + subclass : '') + ' : ' + result.instance;
        //console.log(this.addinstance);
        this.mainService.setinstances(this.route.snapshot.params.gb, this.addinstance, (result.comment ? result.comment : " "))
          .subscribe(
            data => {
              this.mainchild.getmainchoices(this.route.snapshot.params.gb);
              //console.log(data);
            },
            error => {
              if (error.error.text == 'Update Instance') {
                this.notifier.notify("success", (JSON.stringify(error.error.text)));
                if (this.systems) {
                  this.systems.forEach(function (value) {
                    if (section == value.Name) {
                      if (value.TheClasses) {
                        value.TheClasses.forEach(function (value) {
                          if (classes == value.Class) {
                            //UI - Add instance of a Class (e.g instance for System : Vehicle)
                            if (!subclass) {
                              if (value.Instance_of_Class) {
                                value.Instance_of_Class.push({ "Instance": result.instance, "Comment": result.comment ? result.comment : " " })
                              }
                              else {
                                value.Instance_of_Class = [];
                                value.Instance_of_Class.push({ "Instance": result.instance, "Comment": result.comment ? result.comment : " " })
                              }
                            }
                            //UI - Add instance of a SubClass (e.g instance for System : Software : Plan)
                            if (subclass) {
                              if (value.Subclass) {
                                value.Subclass.forEach(function (value) {
                                  if (subclass == value.SubClass_name) {
                                    //console.log('Add instance of a SubClass', value.InstanceOfSubclass);

                                    if (value.InstanceOfSubclass) {
                                      value.InstanceOfSubclass.push({ "Instance": result.instance, "Comment": result.comment ? result.comment : " " })
                                    }
                                    else {
                                      value.InstanceOfSubclass = [];
                                      value.InstanceOfSubclass.push({ "Instance": result.instance, "Comment": result.comment ? result.comment : " " })
                                    }
                                  }
                                })
                              }
                            }
                          }
                        });
                      }
                      if (!classes) { if (value.GeneralInstances) { value.GeneralInstances.push({ "Instance": result.instance, "Comment": result.comment ? result.comment : " " }) } }
                      else { value.GeneralInstances = []; value.GeneralInstances.push({ "Instance": result.instance, "Comment": result.comment ? result.comment : " " }); }
                    }
                  })
                }
                this.mainchild.getmainchoices(this.route.snapshot.params.gb);
              } else { this.notifier.notify("warning", (JSON.stringify(error.error.detail))); }
            });
        //this.ngOnInit();
        //Call to update instances

      }

    });
  }
}
